#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <winsock2.h>

#include "vl.h"
#include "types.h"
#include "server.h"
#include "client.h"

#define TEST_NAME "async_connect"

int process_arg(
	IN              const int opt_index,
	IN              const char* equ_ptr,
	IN              const int arr_size,
	IN              const struct VL_usage_descriptor_t* usage_desc_arr);

void print_config(
	IN		long seed);



struct VL_usage_descriptor_t usage_descriptor[] = {
	{
		'h', "help", "",
		"Print this message and exit",
		#define HELP_CMD_CASE                      0
		HELP_CMD_CASE
	},

	{
		's', "server", "",
		"is server (default no)",
		#define SERVER_CMD_CASE                    1
		SERVER_CMD_CASE
	},

	{
		' ', "seed", "SEED",
		"Seed for rand",
		#define SEED_CMD_CASE                      2
		SEED_CMD_CASE
	},

	{
		' ', "ip", "IP",
		"server's IP addr",
		#define IP_CMD_CASE			   3
		IP_CMD_CASE
	},

	{
		' ', "port", "PORT",
		"server's IP port",
		#define PORT_CMD_CASE                      4
		PORT_CMD_CASE
	},

	{
		' ', "num_ports", "PORTS",
		"number of ports to listen on (default 1)",
		#define PORTS_CMD_CASE                     5
		PORTS_CMD_CASE
	},

	{
		' ', "threads", "THREADS",
		"number of threads per port (default 1)",
		#define THREAD_CMD_CASE                    6
		THREAD_CMD_CASE
	},

	{
		' ', "size", "SIZE",
		"message size (default 1k)",
		#define SIZE_CMD_CASE	                   7
		SIZE_CMD_CASE
	},

	{
		' ', "connections", "CONN",
		"number of connections (default 1000)",
		#define CONN_CMD_CASE                      8
		CONN_CMD_CASE
	},

	{
		' ', "outs_conn", "OUTS",
		"number of outstanding connections (default 1)",
		#define OUTS_CMD_CASE			  10
		OUTS_CMD_CASE
	},

	{
		' ', "validate", "",
		"validate data (default no)",
		#define VALIDATE_CMD_CASE		  11
		VALIDATE_CMD_CASE
	},

	{
		' ', "close_outs", "",
		"disconnect with outstanding connections",
		#define CLOSE_CMD_CASE			  13
		CLOSE_CMD_CASE
	},

	{
		' ', "debug", "DEBUG",
		"debug flags",
		#define DEBUG_CMD_CASE			  14
		DEBUG_CMD_CASE
	},

	{
		' ', "same_port", "",
		"same completion ports for all sockets",
		#define SAME_CMD_CASE			  15
		SAME_CMD_CASE
	},
};


struct config_t config = {
	0,		/* Is server */
	0,		/* Random seed */
	"0",		/* Server's IP */
	5000,		/* Server's port num */
	1000,		/* Number of connections */
	1024,		/* Message size */
	1,		/* Number of outstanding connections */
	0,		/* close with outstanding */
	0,		/* Validate data */
	0,		/* Debug flags */
	0,		/* All sockets on same completion ports */
	1,		/* Threads per port */
	1		/* number of ports server is listening on */
};


enum debug_flags my_debug_flags = 0;

int __cdecl main(int argc, char *argv[])
{
	WORD			wVersionRequested;
	WSADATA			wsaData;
	struct VL_random_t	rand_g;
	int			test_result = 1;
	int			rc;


	rc = VL_parse_argv(argc, argv, (sizeof(usage_descriptor)/sizeof(struct VL_usage_descriptor_t)),
		usage_descriptor, process_arg);

	VL_srand(config.seed, &rand_g);

	print_config(VL_get_seed(&rand_g));

	wVersionRequested = MAKEWORD( 2, 2 );

	rc = WSAStartup( wVersionRequested, &wsaData );
	CHECK_VALUE("WSAStartup", rc, 0, goto cleanup);

	if (config.server) {
		rc = server_main(&rand_g);
		CHECK_VALUE("server_main", rc, 0, goto cleanup);
	}
	else {
		rc = client_main(&rand_g);
		CHECK_VALUE("client_main", rc, 0, goto cleanup);
	}

	test_result = 0;

cleanup:

	VL_print_test_status(test_result);

	return test_result;
}


int process_arg(
		IN              const int opt_index,
		IN              const char* equ_ptr,
		IN              const int arr_size,
		IN              const struct VL_usage_descriptor_t* usage_desc_arr)
{
	/* process argument */
	switch (usage_descriptor[opt_index].case_code) {
	case HELP_CMD_CASE:
		VL_usage(1, arr_size, usage_desc_arr);
		exit(1);

	case SERVER_CMD_CASE:
		config.server = 1;
		break;

	case SEED_CMD_CASE:
		config.seed = strtoul( equ_ptr, 0, 0 );
		break;

	case IP_CMD_CASE:
		strcpy(config.ip, equ_ptr);
		break;

	case PORT_CMD_CASE:
		config.portnum = strtoul( equ_ptr, 0, 0 );
		break;       

	case PORTS_CMD_CASE:  
		config.num_ports = strtoul( equ_ptr, 0, 0 );
		break;       

	case CONN_CMD_CASE:
		config.num_connections = strtoul( equ_ptr, 0, 0 );
		break; 

	case THREAD_CMD_CASE:  
		config.threads_per_port = strtoul( equ_ptr, 0, 0 );
		break;       

	case SIZE_CMD_CASE:
		config.message_size = strtoul( equ_ptr, 0, 0 );
		break;

	case OUTS_CMD_CASE:
		config.num_outs_connections = strtoul( equ_ptr, 0, 0 );
		break;

	case CLOSE_CMD_CASE:
		config.close_with_outs = 1;
		break;
	
	case DEBUG_CMD_CASE:
		config.debug_flags = strtoul( equ_ptr, 0, 0 );
		my_debug_flags = config.debug_flags;
		break;

	case VALIDATE_CMD_CASE:
		config.validate = 1;
		break;

	case SAME_CMD_CASE:
		config.same_ports = 1;
		break;

	default:
		VL_MISC_ERR(("unknown parameter is the switch %s\n",equ_ptr));
		exit(2);
	}

	return 0;
}

void print_config(
		  IN		long seed)
{
	VL_MISC_TRACE(("-----------------------------------------"));
	VL_MISC_TRACE(("Test:                               %s", TEST_NAME));
	VL_MISC_TRACE(("Is Server:                          %s", config.server ? "YES" : "NO"));
	VL_MISC_TRACE(("Seed:                               %d", seed));
	VL_MISC_TRACE(("IP:                                 %s", config.ip));
	VL_MISC_TRACE(("Port Number:                        %d", config.portnum));
	VL_MISC_TRACE(("Number of Ports:                    %d", config.num_ports));
	VL_MISC_TRACE(("Threads per Port:                   %d", config.threads_per_port));
	VL_MISC_TRACE(("Number of Connections:              %d", config.num_connections));
	VL_MISC_TRACE(("Number of Outstanding Connections:  %d", config.num_outs_connections));
	VL_MISC_TRACE(("Message Size:                       %d", config.message_size));
	VL_MISC_TRACE(("All Sockets on Same Ports           %s", config.same_ports ? "YES" : "NO"));
	VL_MISC_TRACE(("Disconnect with Outstanding         %s", config.close_with_outs ? "YES" : "NO"));
	VL_MISC_TRACE(("Validate Data                       %s", config.validate ? "YES" : "NO"));
	VL_MISC_TRACE(("Debug:                              %d", config.debug_flags));
	VL_MISC_TRACE(("-----------------------------------------"));
}


