#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <winsock2.h>
#include <MSWSock.h>

#include "vl.h"
#include "types.h"

extern enum debug_flags my_debug_flags;
extern struct config_t config;

int init_socket_data(
	IN struct VL_random_t *rang_t, 
	IN struct socket_data *socket_array)
{
	struct sockaddr_in      serv_addr;
	struct sockaddr_in      cli_addr;
	int			clilen;
	int			i;
	int			rc;

	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = inet_addr(config.ip);
	clilen = sizeof(cli_addr);

	for (i = 0; i < config.num_ports; ++i)
		socket_array[i].socket = 0;

	for (i = 0; i < config.num_ports; ++i) {
		serv_addr.sin_port = htons((u_short)config.portnum + i);

		socket_array[i].socket = socket(AF_INET, SOCK_STREAM, 0);
		CHECK_NOT_EQUAL("socket", socket_array[i].socket, INVALID_SOCKET, return -1);

		rc = bind(socket_array[i].socket, (struct sockaddr *) &serv_addr, sizeof(serv_addr));
		CHECK_NOT_EQUAL("bind", rc, SOCKET_ERROR, return -1);

		rc = listen(socket_array[i].socket, SOMAXCONN);
		CHECK_NOT_EQUAL("listen", rc, SOCKET_ERROR, return -1);

		socket_array[i].done = 0;
		socket_array[i].idx = i;
		socket_array[i].key = VL_random(rang_t, 0xFFFFFFFF);
		socket_array[i].next = ACCEPTEX;
		socket_array[i].num_accepted = 0;
		socket_array[i].num_posted = 0;
		socket_array[i].num_connections = (config.num_connections / config.num_outs_connections) * config.num_outs_connections;
		socket_array[i].rand_t = *rang_t;

		InitializeCriticalSection(&socket_array[i].socket_sync);
	}

	return 0;
}

int init_comp_ports(
	IN	struct comp_port_data_t *port_array,
	IN	struct socket_data *socket_array,
	IN	int num_comp_ports)
{
	HANDLE temp_handle;
	int i;
	int socket_per_port = config.num_ports / num_comp_ports;
	int port_idx = 0;
	unsigned long temp_key = 0;

	for (i = 0; i < num_comp_ports; ++i) {
		port_array[i].port = CreateIoCompletionPort(INVALID_HANDLE_VALUE, NULL, temp_key, config.threads_per_port);
		CHECK_NOT_EQUAL("CreateIoCompletionPort", port_array[i].port, NULL, return -1);

		InitializeCriticalSection(&port_array[i].ret_val_lock);
		port_array[i].ret_val = 0;
	}

	for (i = 0; i < config.num_ports; ++i) {
		temp_handle = CreateIoCompletionPort((HANDLE)socket_array[i].socket, port_array[port_idx].port, 
						      socket_array[i].key, config.threads_per_port);
		CHECK_NOT_EQUAL("CreateIoCompletionPort", temp_handle, NULL, return -1);

		port_idx++;
		if (port_idx == num_comp_ports)
			port_idx = 0;
	}

	return 0;
}

int init_overlapped_data(
	IN struct socket_data *socket_array, 
	IN struct overlapped_data_t *overlapped_array)
{
	int i;
	int j;

	for (i = 0; i < config.num_ports * config.num_outs_connections; ++i) {
		overlapped_array[i].data_buf = NULL;
		memset(&overlapped_array[i].overlapped, 0, sizeof(overlapped_array[i].overlapped));
	}

	for (i = 0; i < config.num_ports; ++i) {
		for (j = 0; j < config.num_outs_connections; ++j) {
			overlapped_array[j + i * config.num_outs_connections].socket = socket(AF_INET, SOCK_STREAM, 0);
			CHECK_NOT_EQUAL("socket", overlapped_array[i].socket, INVALID_SOCKET, return -1);

			overlapped_array[j + i * config.num_outs_connections].bind_socket = &socket_array[i];
			overlapped_array[j + i * config.num_outs_connections].data_buf_len = 
								config.message_size + LOCALADDRESSLENGTH + DWREMOTEADDRESSLENGTH;
			overlapped_array[j + i * config.num_outs_connections].data_buf = 
				malloc(overlapped_array[j + i * config.num_outs_connections].data_buf_len * sizeof(char));
			CHECK_NOT_EQUAL("malloc", overlapped_array[j + i * config.num_outs_connections].data_buf, NULL, return -1);

			memset(overlapped_array[j + i * config.num_outs_connections].data_buf, 0, 
				overlapped_array[j + i * config.num_outs_connections].data_buf_len);
		}
	}

	return 0;
}

void* worker_thread(
		    IN	void* data)
{
	struct comp_port_data_t		*comp_data = (struct comp_port_data_t *)data;
	HANDLE				completion_port = comp_data->port;
	struct overlapped_data_t	*overlapped = NULL;
	int				len;
	int				temp_len;
	unsigned long			key;
	int				result = -1;
	int				rc;

	while (TRUE) {
		overlapped = NULL;
		rc = GetQueuedCompletionStatus(completion_port, &len, (PULONG_PTR)&key, (LPOVERLAPPED *)&overlapped, INFINITE);
		CHECK_NOT_EQUAL("GetQueuedCompletionStatus", rc, 0, goto cleanup);
		CHECK_VALUE("key", key, overlapped->bind_socket->key, goto cleanup);

		EnterCriticalSection(&overlapped->bind_socket->socket_sync);
		overlapped->bind_socket->num_accepted++;
		if (overlapped->bind_socket->num_accepted == overlapped->bind_socket->num_connections)
			overlapped->bind_socket->done = 1;
		LeaveCriticalSection(&overlapped->bind_socket->socket_sync);

		if (config.validate) {
			rc = validate_buffer_data(overlapped->data_buf, len, (char)overlapped->bind_socket->idx);
			CHECK_VALUE("validate_buffer_data", rc, 0, goto cleanup);

			memset(overlapped->data_buf, 0, len);
		}
		
		memset(&overlapped->overlapped, 0, sizeof(overlapped->overlapped));

		rc = closesocket(overlapped->socket);
		CHECK_NOT_EQUAL("closesocket", rc, SOCKET_ERROR, goto cleanup);

		EnterCriticalSection(&overlapped->bind_socket->socket_sync);
		if (overlapped->bind_socket->done) {
			DEBUG(MESSAGE, my_debug_flags, 
				("socket %d idx %d done", overlapped->bind_socket->socket, overlapped->bind_socket->idx));
			if (!config.same_ports) {
				LeaveCriticalSection(&overlapped->bind_socket->socket_sync);
				break;
			}
		}
		else if (overlapped->bind_socket->num_posted < overlapped->bind_socket->num_connections || config.close_with_outs) {
			overlapped->socket = socket(AF_INET, SOCK_STREAM, 0);
			CHECK_NOT_EQUAL("socket", overlapped->socket, INVALID_SOCKET, goto cleanup);

			rc = AcceptEx(overlapped->bind_socket->socket, overlapped->socket, overlapped->data_buf, 
				config.message_size, LOCALADDRESSLENGTH, DWREMOTEADDRESSLENGTH, &temp_len, (LPOVERLAPPED)overlapped);
			if (rc == FALSE && GetLastError() != ERROR_IO_PENDING) {
				ErrorExit("AcceptEx");
				VL_MISC_ERR(("AcceptEx failed"));
				goto cleanup;
			}
			overlapped->bind_socket->num_posted++;
		}

		LeaveCriticalSection(&overlapped->bind_socket->socket_sync);	
	}

	result = 0;
cleanup:

	if (result) {
		EnterCriticalSection(&comp_data->ret_val_lock);
		comp_data->ret_val = result;
		LeaveCriticalSection(&comp_data->ret_val_lock);
	}

	return NULL;
}

int wait_for_test_end(
	IN	struct VL_thread_t *threads,
	IN	int num_threads,
	IN	struct comp_port_data_t *port_array,
	IN	int num_comp_ports,
	IN	struct socket_data *socket_array)
{
	int i;

	while (TRUE) {
		int temp;
		int num_done = 0;

		for (i = 0; i < config.num_ports; ++i) {
			EnterCriticalSection(&socket_array->socket_sync);
			num_done += socket_array->done;
			LeaveCriticalSection(&socket_array->socket_sync);
		}

		if (num_done == config.num_ports) {
			DEBUG(MESSAGE, my_debug_flags, ("All sockets done"));
			break;
		}

		for (i = 0; i < num_comp_ports; ++i) {
			EnterCriticalSection(&port_array[i].ret_val_lock);
			temp = port_array[i].ret_val;
			LeaveCriticalSection(&port_array[i].ret_val_lock);

			CHECK_VALUE("ret val", temp, 0, return -1);
		}

		VL_msleep(100);
	}

	for (i = 0; i < num_comp_ports * config.threads_per_port; ++i)
		VL_thread_kill(&threads[i]);

	for (i = 0; i < num_comp_ports; ++i) {
		CHECK_VALUE("ret val", port_array[i].ret_val, 0, return -2);
	}

	return 0;
}

int server_main(
	IN	struct VL_random_t *rand_t)
{
	struct socket_data		*socket_array = NULL;
	struct overlapped_data_t	*overlapped_array = NULL;
	struct comp_port_data_t		*port_array = NULL;
	struct VL_thread_t		*threads = NULL;
	int				num_comp_ports;
	int				temp_len;
	int				i;
	int				result = -1;
	int				rc;

	if (config.same_ports)
		num_comp_ports = config.num_ports / VL_range(rand_t, 1, config.num_connections);
	else
		num_comp_ports = config.num_ports;

	if (num_comp_ports == 0)
		num_comp_ports = 1;

	port_array = malloc(num_comp_ports * sizeof(struct comp_port_data_t));
	CHECK_NOT_EQUAL("malloc", port_array, NULL, goto cleanup);

	socket_array = malloc(config.num_ports * sizeof(struct socket_data));
	CHECK_NOT_EQUAL("malloc", socket_array, NULL, goto cleanup);

	overlapped_array = malloc(config.num_ports * config.num_outs_connections * sizeof(struct overlapped_data_t));
	CHECK_NOT_EQUAL("malloc", overlapped_array, NULL, goto cleanup);

	rc = init_socket_data(rand_t, socket_array);
	CHECK_VALUE("init_socket_data", rc, 0, goto cleanup);

	rc = init_comp_ports(port_array, socket_array, num_comp_ports);
	CHECK_VALUE("init_comp_ports", rc, 0, goto cleanup);

	rc = init_overlapped_data(socket_array, overlapped_array);
	CHECK_VALUE("init_overlapped_data", rc, 0, goto cleanup);

	for (i = 0; i < config.num_ports * config.num_outs_connections; ++i) {
		rc = AcceptEx(overlapped_array[i].bind_socket->socket, overlapped_array[i].socket, overlapped_array[i].data_buf, 
			config.message_size, LOCALADDRESSLENGTH, DWREMOTEADDRESSLENGTH, &temp_len, (LPOVERLAPPED)&overlapped_array[i]);
		if (rc == FALSE && GetLastError() != ERROR_IO_PENDING) {
			ErrorExit("AcceptEx");
			VL_MISC_ERR(("AcceptEx failed"));
			goto cleanup;
		}
	}

	threads = malloc(num_comp_ports * config.threads_per_port * sizeof(struct VL_thread_t));
	CHECK_NOT_EQUAL("malloc", threads, NULL, goto cleanup);

	for (i = 0; i < num_comp_ports * config.threads_per_port; ++i) {
		rc = VL_thread_start(&threads[i], 0, worker_thread, &port_array[i / config.threads_per_port]);
		CHECK_VALUE("VL_thread_start", rc, 0, goto cleanup);
	}

	rc = wait_for_test_end(threads, num_comp_ports * config.threads_per_port, port_array, num_comp_ports, socket_array);
	CHECK_VALUE("wait_for_test_end", rc, 0, goto cleanup);

	result = 0;
cleanup:
	
	if (overlapped_array) {
		for (i = 0; i < config.num_ports * config.num_outs_connections; ++i) {
			if (overlapped_array[i].data_buf)
				free(overlapped_array[i].data_buf);
		}
		free(overlapped_array);
	}

	if (socket_array) {
		for (i = 0; i < config.num_ports; ++i) {
			if (socket_array[i].socket) {
				rc = closesocket(socket_array[i].socket);
				CHECK_VALUE("closesocket", rc, 0, result = -1);
			}
		}
		free(socket_array);
	}

	if (port_array)
		free(port_array);

	if (threads)
		free(threads);

	return result;
}