#ifndef _TYPES_H_
#define _TYPES_H_

#define TIMEOUT 5000
#define NUM_TIMEOUTS 20
#define LOCALADDRESSLENGTH 26 // 26 is the minimum
#define DWREMOTEADDRESSLENGTH 22 //(0 => 10114) 16=>ERROR_INSUFFICIENT_BUFFER, 22 is the minimum

#define VL_IP_STR_LENGTH 15



const char *winsockerror(
	IN      u_int32_t e);

void ErrorExit(
	IN	const char *funct);

void set_buffer_data(
	IN	char* buf,
	IN	int len,
	IN	char val);

int validate_buffer_data(
	IN	char* buf,
	IN	int len,
	IN	char comp);

enum operation { ACCEPTEX };

enum debug_flags {
	ENTER_FUNCT	= 1,
	SYNC_SIDES	= (1 << 1),
	SEND_DATA	= (1 << 2),
	RECV_DATA	= (1 << 3),
	MESSAGE		= (1 << 4)
};

struct config_t {
	int			server;
	unsigned int		seed;
	char			ip[VL_IP_STR_LENGTH];
	int			portnum;
	int			num_connections;
	int			message_size;
	int			num_outs_connections;
	int			close_with_outs;
	int			validate;
	u_int32_t		debug_flags;
	int			same_ports; // same completion ports for all clients
	int			threads_per_port;
	int			num_ports;
};


struct socket_data {
	SOCKET			socket;
	unsigned long		key;
	int			idx;
	int			num_accepted;
	int			num_posted;
	int			num_connections;
	int			done;
	enum operation		next;
	struct VL_random_t	rand_t;
	CRITICAL_SECTION	socket_sync;
};

struct overlapped_data_t {
	WSAOVERLAPPED		overlapped;
	SOCKET			socket;
	char			*data_buf;
	int			data_buf_len;
	struct socket_data	*bind_socket;
};

struct comp_port_data_t {
	HANDLE			port;
	int			ret_val;
	CRITICAL_SECTION	ret_val_lock;
};


#define CHECK_NOT_EQUAL(verb, act_val, exp_val, cmd) if ((act_val) == (exp_val)) {					\
								ErrorExit(verb);					\
								VL_MISC_ERR(("%s failed", verb));			\
								cmd;							\
							}

#define CHECK_VALUE(verb, act_val, exp_val, cmd) if ((act_val) != (exp_val)) {						\
							VL_MISC_ERR(("Error in %s, expected value %d, actual value %d",	\
							(verb), (exp_val), (act_val))); 				\
							cmd; 								\
						}

#define DEBUG(what, debug_mask, print) if ((what) & (debug_mask)) {							\
						VL_MISC_TRACE(print);							\
					}

#endif