/*
 * Copyright (c) 2005 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: vl_rand.c 114 2005-09-19 14:00:17Z dotanb $ 
 * 
 */

#include <vl_rand.h>
#include <vl_os.h>

/***************************************************
* Function: VL_srand
***************************************************/
unsigned long VL_srand(
    IN                  unsigned long seed,
    INOUT               struct VL_random_t* rand_p)
{
    if (seed == 0) {
        struct VL_time_t time;

        VL_gettimeofday(&time);

        rand_p->rseed = time.sec*1000000 + time.msec * 1000;
    } else
        rand_p->rseed = seed;

    return VL_get_seed(rand_p);
}

/***************************************************
* Function: VL_get_seed
***************************************************/
unsigned long VL_get_seed(
    IN                  const struct VL_random_t* rand_p)
{
    return rand_p->rseed;
}

/***************************************************
* Function: VL_rand
***************************************************/
unsigned int VL_random(
    INOUT               struct VL_random_t* rand_p,
    IN                  unsigned long end)
{
    unsigned int rand_value;

    rand_p->rseed = ((rand_p->rseed * 134775813 + 1) & 0xffffffff);
    rand_value = ((rand_p->rseed >> 16)*end + ((rand_p->rseed & 0xffff)*end >> 16)) >> 16;

    return rand_value;
}

/***************************************************
* Function: VL_range
***************************************************/
unsigned int VL_range(
    INOUT               struct VL_random_t* rand_p,
    IN                  unsigned long start,
    IN                  unsigned long end)
{
    /* if the values are equal, return one of them */
    if (start == end)
        return start;

    return (VL_random(rand_p, end - start + 1) + start);
}

