/*
 * Copyright (c) 2005 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: vl_trace.c 114 2005-09-19 14:00:17Z dotanb $ 
 * 
 */

#include <stdlib.h>
#include <vl_types.h>
#include <vl_trace.h>

/***************************************************
* Function: VL_debug_flags
***************************************************/
unsigned long VL_debug_flags()
{
    static unsigned long flags = 0;
    static int first_time = 1;
    
    if (first_time) {
        const char *envname = VL_ENV_NAME;
        char *env = getenv(envname);

        if (env) {
            flags = strtoul(env, 0, 0);
        }

        first_time = 0;
    }

    return flags;
}

/***************************************************
* Function: VL_print_test_status
***************************************************/
void VL_print_test_status(
    IN                  int test_status)
{
    const char* color_fmt;
    const char* msg;;


    if (test_status == 0) {
        color_fmt = COLOR_GREEN;
        msg = "[TEST PASSED]";
    } else {
        color_fmt = COLOR_RED;
        msg = "[TEST FAILED]";
    }

    VL_PRINTF("\n\n");
    VL_PRINTF("%s\t-------------------------------------------\n"COLOR_BLACK, color_fmt);
    VL_PRINTF("%s\t\t\t%s\n"COLOR_BLACK, color_fmt, msg);
    VL_PRINTF("%s\t-------------------------------------------\n"COLOR_BLACK, color_fmt);
    VL_PRINTF("\n\n");
}

